
/** @file Bsum.h
 *
 * Interface to Bsums 
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_Bsum_H__
#define __NESTEDSUMS_Bsum_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// Bsums arise from Hoelder convolutions.
  /**
   *
   * A Bsum is defined by
   *  \f[
   *       \sum\limits_{i_1=n+1}^N
   *       \sum\limits_{i_2=i_1+1}^N ...
   *       \sum\limits_{i_k=i_{k-1}+1}^N
   *        \frac{x_1^{i_1}}{i_1^{m_1}}
   *        \frac{x_2^{i_2}}{i_2^{m_2}} ...
   *        \frac{x_k^{i_k}}{i_k^{m_k}}
   *  \f]
   * This is equivalent to
   * \f[
   *     \left( x_k^+ \right)^{m_k} \left( x_{k-1}^+ \right)^{m_{k-1}} ... 
   *     \left( x_1^+ \right)^{m_1} 
   *     \frac{x_k}{1-x_k} \frac{x_{k-1}x_k}{1-x_{k-1}x_k} ...
   *     \frac{x_1 ... x_k}{1-x_1 ... x_k}
   *     \left( x_1 ... x_k \right)^n 
   *  \f]
   * up to terms of the form
   * \f[
   * \left( x^+ \right)^m \frac{x}{1-x} x^N = \sum\limits_{i=N+1}^\infty \frac{x^i}{i^m}
   * \f]
   *
   */
  class Bsum : public basic
    {

      GINAC_DECLARE_REGISTERED_CLASS(Bsum, basic)

	// ctors
    public:
      explicit Bsum(unsigned ti);
      Bsum(const ex & nc, const ex & llc);
      Bsum(unsigned ti, const ex & nc, const ex & llc);

	// functions overriding virtual functions from bases classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;
      ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;

    protected:
      ex simplify_ncmul(const exvector & v) const;
      ex derivative(const symbol & s) const;
      unsigned return_type(void) const { return return_types::noncommutative; }
      unsigned return_type_tinfo(void) const { return TINFO_Bsum; }
      unsigned calchash(void) const;

	// new virtual functions which can be overridden by derived classes
    public :
      virtual ex convert_to_Ssum_exvector(const exvector & Z1) const;

      virtual ex expand_members(int level = 0) const;

      // non-virtual functions
    public :
      ex get_index(void) const;
      ex get_letter_list(void) const;
      unsigned get_depth(void) const;

    protected:

      // friends :
      friend ex convert_Bsum_to_Ssum(const ex & C);
      friend ex convert_Bsum_to_Zsum(const ex & C);

	// member variables :

    protected:

      ex n;
      ex letter_list;

      static unsigned precedence;
    };

  // inline functions

  /**
   *
   * Returns the upper summation limit.
   *
   */
inline ex Bsum::get_index(void) const
  {
    return n;
  }

  /**
   *
   * Returns the letter_list.
   *
   */
inline ex Bsum::get_letter_list(void) const
  {
    return letter_list;
  }

  /**
   *
   * Returns the depth.
   *
   */
inline unsigned Bsum::get_depth(void) const
  {
    return letter_list.nops();
  }

  // utility functions
inline const Bsum &ex_to_Bsum(const ex &e)
  {
    return static_cast<const Bsum &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_Bsum(const ex & nc, const ex & llc)
  {
    return (new Bsum(nc,llc))->setflag(status_flags::dynallocated);
  }

 ex create_Bsum_from_exvector(const ex & nc, const exvector & v);

 ex convert_Bsum_to_Ssum(const ex & C);
 ex convert_Bsum_to_Zsum(const ex & C);

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_Bsum_H__
