
/** @file Csum.h
 *
 * Interface to Csums 
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_Csum_H__
#define __NESTEDSUMS_Csum_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// Csums involve a conjugation.
  /**
   *
   *  A Csum is defined by
   *  \f[
   *     \mbox{} - \sum\limits_{i=1}^n 
   *       \left( \begin{array}{c} n \\ i \\ \end{array} \right)
   *       \left( -1 \right)^i
   *       \frac{x_0^i}{i^{m_0}}
   *       \sum\limits_{i_1=i+1}^N
   *       \sum\limits_{i_2=i_1+1}^N ...
   *       \sum\limits_{i_k=i_{k-1}+1}^N
   *        \frac{x_1^{i_1}}{i_1^{m_1}}
   *        \frac{x_2^{i_2}}{i_2^{m_2}} ...
   *        \frac{x_k^{i_k}}{i_k^{m_k}}
   *  \f]
   *  This is equivalent to
   *  \f[
   *     \left( x_k^+ \right)^{m_k} \left( x_{k-1}^+ \right)^{m_{k-1}} ... 
   *     \left( x_1^+ \right)^{m_1} \left( x_0^+ \right)^{m_0}
   *     \frac{x_k}{1-x_k} \frac{x_{k-1}x_k}{1-x_{k-1}x_k} ...
   *     \frac{x_1 ... x_k}{1-x_1 ... x_k}
   *     \left[ 1 - \left( 1 - x_0 x_1 ... x_k \right)^n \right]
   *  \f]
   * up to terms of the form
   * \f[
   * \left( x^+ \right)^m \frac{x}{1-x} x^N = \sum\limits_{i=N+1}^\infty \frac{x^i}{i^m}
   * \f]
   *  Csums can be expressed in terms of Zsums or Ssums.
   *
   *
   * Basic formulae used here are:
   * \f[
   *    x_1^+ \left[ 1 - \left( 1 - x_1 x_2 \right)^n \right] = 
   *      \sum\limits_{i=1}^n \frac{1}{i} \left[ 1 - \left( 1 - x_1 x_2 \right)^i \right] 
   * \f]
   * \f[
   *    x_1^+ \frac{x_1 x_2}{1-x_1 x_2} \left[ 1 - \left( 1 - x_0 x_1 x_2 \right)^n 
   *          \mbox{} - \left( x_1 x_2 \right)^N \left( 1 - \left( 1 - x_0 \right)^n \right) \right] = 
   * \f] \f[
   *      = \mbox{} - \left( 1 - x_0 \right)^n \sum\limits_{i=1}^n \frac{1}{i} 
   *                \left( \frac{1}{1-x_0} \right)^i \left[ 1 - \left( 1 - x_0 x_1 x_2 \right)^i \right]
   *    + \left( 1 - \left( 1 - x_0 \right)^n \right) \sum\limits_{i=1}^N \frac{\left(x_1 x_2 \right)^i}{i} 
   * \f]
   * \f[
   *    x_1^+ \frac{x_1 x_2}{1-x_1 x_2} \left[ 1 - \left( 1 - x_1 x_2 \right)^n 
   *          \mbox{} - \left( x_1 x_2 \right)^N \right] = 
   *      \mbox{} - \frac{1}{n} 
   *                \left[ 1 - \left( 1 - x_1 x_2 \right)^n \right]
   *    + \sum\limits_{i=1}^N \frac{\left(x_1 x_2 \right)^i}{i} 
   * \f]
   *
   *
   */
  class Csum : public basic
    {

      GINAC_DECLARE_REGISTERED_CLASS(Csum, basic)

	// ctors
    public:
      explicit Csum(unsigned ti);
      Csum(const ex & nc, const ex & llc);
      Csum(unsigned ti, const ex & nc, const ex & llc);

	// functions overriding virtual functions from bases classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;
      ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;

    protected:
      ex simplify_ncmul(const exvector & v) const;
      ex derivative(const symbol & s) const;
      unsigned return_type(void) const { return return_types::noncommutative; }
      unsigned return_type_tinfo(void) const { return TINFO_Csum; }
      unsigned calchash(void) const;

	// new virtual functions which can be overridden by derived classes
    public :
      virtual ex convert_to_Ssum_exvector(const exvector & Z1) const;

      virtual ex expand_members(int level = 0) const;

    protected:
      virtual ex decrease_letter_degree(int m) const;
      virtual ex move_x0_into_x1(void) const;
      virtual ex move_x1_into_x2(void) const;
      virtual ex remove_x0(void) const;
      virtual ex remove_x0_and_x1(void) const;
      virtual ex cast_to_Bsum(void) const;

      // non-virtual functions
    public :
      ex get_index(void) const;
      ex get_letter_list(void) const;
      unsigned get_depth(void) const;

      // friends :
      friend ex convert_Csum_to_Ssum(const ex & C);
      friend ex convert_Csum_to_Zsum(const ex & C);

	// member variables :

    protected:

      ex n;
      ex letter_list;

      static unsigned precedence;
    };

  // inline functions

  /**
   *
   * Returns the upper summation limit.
   *
   */
inline ex Csum::get_index(void) const
  {
    return n;
  }

  /**
   *
   * Returns the letter_list.
   *
   */
inline ex Csum::get_letter_list(void) const
  {
    return letter_list;
  }

  /**
   *
   * Returns the depth.
   *
   */
inline unsigned Csum::get_depth(void) const
  {
    return letter_list.nops();
  }

  // utility functions
inline const Csum &ex_to_Csum(const ex &e)
  {
    return static_cast<const Csum &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_Csum(const ex & nc, const ex & llc)
  {
    return (new Csum(nc,llc))->setflag(status_flags::dynallocated);
  }

 ex create_Csum_from_exvector(const ex & nc, const exvector & v);

 ex convert_Csum_to_Ssum(const ex & C);
 ex convert_Csum_to_Zsum(const ex & C);

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_Csum_H__
