
/** @file Ssum.h
 *
 * Interface to Ssums 
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_SSUM_H__
#define __NESTEDSUMS_SSUM_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// Ssums form an algebra.
  /**
   *
   *  A Ssum is recursively defined by
   *  \f[
   *     S(n;m_1,...,m_k;x_1,...,x_k) = \sum\limits_{i=1}^n \frac{x_1^i}{i^{m_1}} S(i;m_2,...,m_k;x_2,...,x_k)
   *  \f]
   *  with 
   *  \f[
   *     S(n) = 1
   *  \f]
   *  for \f$ n > 0 \f$ and
   *  \f[
   *     S(n) = 0
   *  \f]
   * for \f$ n \le 0 \f$.
   *
   *  Ssums can be converted to Zsums (and vice versa).
   *
   *  All letters should have their index set to _default_index.
   *
   */
  class Ssum : public basic
    {
      // Csum accesses the methods
      //   multiply_last_letter
      //   increases_las_letter
      //   append_letter
      friend class Csum;

      GINAC_DECLARE_REGISTERED_CLASS(Ssum, basic)

	// ctors
    public:
      explicit Ssum(unsigned ti);
      explicit Ssum(const ex & nc);
      Ssum(const ex & nc, const ex & llc);
      Ssum(unsigned ti, const ex & nc, const ex & llc);

	// functions overriding virtual functions from bases classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;
      ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;

    protected:
      ex simplify_ncmul(const exvector & v) const;
      ex derivative(const symbol & s) const;
      unsigned return_type(void) const { return return_types::noncommutative; }
      unsigned return_type_tinfo(void) const { return TINFO_Ssum; }
      unsigned calchash(void) const;

	// new virtual functions which can be overridden by derived classes
    public :
      virtual ex convert_to_Zsum_exvector(const exvector & Z0, const exvector & Z1) const;
      virtual ex shuffle_exvector(const exvector & Z0, const exvector & Z1, const exvector & Z2) const;

      virtual ex set_index(const ex & i) const;
      virtual ex shift_plus_one(void) const;
      virtual ex shift_minus_one(void) const;
      virtual ex adjust_upper_limit_downwards(const ex & i) const;
      virtual ex adjust_upper_limit_upwards(const ex & i) const;
      virtual ex adjust_upper_limit_plus_one(void) const;
      virtual ex index_eq_one(void) const;

      virtual ex get_head(int k) const;
      virtual ex get_tail(int k) const;
      virtual int flag_remove_negative_degrees(void) const;
      virtual ex remove_negative_degrees(void) const;

      virtual ex expand_members(int level = 0) const;
      virtual ex eval_explicit(int level = 0) const;

      virtual ex get_first_letter(void) const;
      virtual ex remove_first_letter(void) const;
      virtual ex remove_first_letter(const ex & nc) const;
      virtual ex prepend_letter(const ex & lc) const;
      virtual ex prepend_letter(const ex & nc, const ex & lc) const;
      virtual ex append_letter(const ex & lc) const;
      virtual ex append_letter_list(const ex & lc) const;

    protected:
      virtual ex cast_to_Zsum(void) const;

      virtual ex multiply_letter_with_last_letter(const ex & lc) const;
      virtual ex multiply_letter_with_first_letter(const ex & lc) const;
      virtual ex concat_two_sums(const ex & Z1, const ex & Z2) const;

      // non-virtual functions
    public :
      ex get_index(void) const;
      ex get_letter_list(void) const;
      unsigned get_depth(void) const;
      ex get_weight(void) const;

      // friends :
      friend ex shuffle_Ssum(const ex & Z1, const ex & Z2);
      friend ex convert_Ssum_to_Zsum(const ex & Z1);
      friend ex remove_trivial_Ssum(const ex & expr);

	// member variables :

    protected:

      ex n;
      ex letter_list;

      static unsigned precedence;
    };

  // inline functions

  /**
   *
   * Returns the upper summation limit.
   *
   */
inline ex Ssum::get_index(void) const
  {
    return n;
  }

  /**
   *
   * Returns the letter_list.
   *
   */
inline ex Ssum::get_letter_list(void) const
  {
    return letter_list;
  }

  /**
   *
   * Returns the depth.
   *
   */
inline unsigned Ssum::get_depth(void) const
  {
    return letter_list.nops();
  }

  // utility functions
inline const Ssum &ex_to_Ssum(const ex &e)
  {
    return static_cast<const Ssum &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_Ssum(const ex & nc, const ex & llc)
  {
    return (new Ssum(nc,llc))->setflag(status_flags::dynallocated);
  }

 ex shuffle_Ssum(const ex & Z1, const ex & Z2);

 ex convert_Ssum_to_Zsum(const ex & Z1);

 ex create_Ssum_from_exvector(const ex & nc, const exvector & v);

 ex Ssum_to_Zsum(const ex & expr);

 ex shift_upper_limit_plus_one_for_Ssum(const ex & expr);

 ex remove_negative_degrees_from_Ssum(const ex & expr);

 ex remove_trivial_Ssum(const ex & expr);

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_SSUM_H__
