
/** @file basic_letter.h
 *
 * Interface to basic_letter
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_BASIC_LETTER_H__
#define __NESTEDSUMS_BASIC_LETTER_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// A basic_letter is an element of an alphabet.
  /**
   *
   *  The model for a basic_letter is
   *  \f[
   *     \frac{x^i}{(i+c)^m}
   *  \f]
   *  Here \f$x\f$ is the scale, \f$m\f$ the degree,
   *  \f$c\f$ the offset and \f$i\f$ a parameter (usually a summation index).
   *  \f$i\f$ does not need to be specified. In this case the index is set
   *  to the default index.
   *
   *  Two basic_letters can be multiplied ("concatenated") to form 
   *  a new basic_letter.
   *
   */
  class basic_letter : public basic
    {
      GINAC_DECLARE_REGISTERED_CLASS(basic_letter, basic)

	// ctors
    public:
      explicit basic_letter(unsigned ti);
      basic_letter(const ex & l, const ex & d, const ex & o);
      basic_letter(const ex & l, const ex & d, const ex & o, const ex & i);
      basic_letter(unsigned ti, const ex & l, const ex & d, const ex & o);
      basic_letter(unsigned ti, const ex & l, const ex & d, const ex & o, const ex & i);

	// functions overriding virtual functions from base classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;
      ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;

    protected:
      ex simplify_ncmul(const exvector & v) const;
      ex derivative(const symbol & s) const;
      unsigned return_type(void) const { return return_types::noncommutative; }
      unsigned return_type_tinfo(void) const { return TINFO_basic_letter; }
      unsigned calchash(void) const;

	// new virtual functions which can be overridden by derived classes
    public:
      virtual ex concat(const ex & l) const;
      virtual ex eval_explicit(int level = 0) const;
      virtual ex expand_members(int level = 0) const;

    protected:
      virtual ex concat_speedy(const ex & l) const;

	// non-virtual functions 
    public:
       ex get_scale(void) const;
       ex get_degree(void) const;
       ex get_offset(void) const;
       ex get_index(void) const;

       ex set_index(const ex & i) const;
       ex forget_index(void) const;
       ex shift_index(const ex & new_index) const;
       ex index_eq_one(void) const;

       ex shift_plus_one(void) const;
       ex shift_minus_one(void) const;
       ex degree_minus_one(void) const;
       ex degree_plus_one(void) const;

    protected:

        // friends :
       friend ex concat(const basic_letter & l1, const basic_letter & l2);

	// member variables :

    protected:

      ex scale;
      ex degree;
      ex offset;
      ex index;

      static unsigned precedence;
    };

  // inline functions

  /**
   *
   * Returns the scale
   *
   */
inline ex basic_letter::get_scale(void) const
  {
    return scale;
  }

  /**
   *
   * Returns the degree
   *
   */
inline ex basic_letter::get_degree(void) const
  {
    return degree;
  }

  /**
   *
   * Returns the offset
   *
   */
inline ex basic_letter::get_offset(void) const
  {
    return offset;
  }

  /**
   *
   * Returns the index
   *
   */
inline ex basic_letter::get_index(void) const
  {
    return index;
  }

  // utility functions
inline const basic_letter &ex_to_basic_letter(const ex &e)
  {
    return static_cast<const basic_letter &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_basic_letter(const ex & l, const ex & d, const ex & o)
  {
    return (new basic_letter(l,d,o))->setflag(status_flags::dynallocated);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_basic_letter(const ex & l, const ex & d, const ex & o, const ex & i)
  {
    return (new basic_letter(l,d,o,i))->setflag(status_flags::dynallocated);
  }

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_BASIC_LETTER_H__
