
/** @file classical_polylog.cc
 *
 * Implementation of classical polylogs 
 *
 */

/*
 *  gTybalt Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <stdexcept>
#include "nestedsums.h"

#ifdef _DEBUG
#include <ctime>
#endif // def _DEBUG

namespace GiNaC {

GINAC_IMPLEMENT_REGISTERED_CLASS(classical_polylog, nielsen_polylog)



  classical_polylog::classical_polylog() : inherited(TINFO_classical_polylog)
  { 
  }

  classical_polylog::classical_polylog(unsigned ti) : inherited(ti)
  { 
  }

  void classical_polylog::destroy(bool call_parent)
  {
    if (call_parent)
      inherited::destroy(call_parent);
  }

  void classical_polylog::copy(const classical_polylog &other)
  {
    inherited::copy(other);
  }

  void classical_polylog::archive(archive_node &node) const
  {
    inherited::archive(node);
  }

  classical_polylog::classical_polylog(const archive_node &node, const lst &sym_lst) : inherited(node, sym_lst),
    Zsum(node, sym_lst)
  {
  }

  ex classical_polylog::unarchive(const archive_node &node, const lst &sym_lst)
  {
    return (new classical_polylog(node, sym_lst))->setflag(status_flags::dynallocated);
  }

  int classical_polylog::compare_same_type(const basic &other) const
  {
    const classical_polylog &o = dynamic_cast<const classical_polylog &>(other);
    int cmpval;

    cmpval = letter_list.compare(o.letter_list);
    if ( cmpval) return cmpval;

    return n.compare(o.n);
  }

  // other ctors

  // ctor with initialization list
  classical_polylog::classical_polylog(const ex & llc) : inherited(TINFO_classical_polylog,llc),
    Zsum(TINFO_classical_polylog,Infinity,llc)
  { }

  classical_polylog::classical_polylog(unsigned ti, const ex & llc) : inherited(ti,llc),
    Zsum(ti,Infinity,llc)
  { }

  // functions overriding virtual functions from bases classes
  void classical_polylog::print(const print_context & c, unsigned level) const
  {
    if ( _print_format ) return multiple_polylog::print(c,level);

    unsigned depth = get_depth();

    // empty sum equals 1
    if (depth == 0) 
      {
	if (is_a<print_latex>(c)) 
	  {
	    if (precedence <= level) c.s << "{(";
	    c.s << "Z(" ;
	    n.print(c);
	    c.s << ")";
	    if (precedence <= level) c.s << ")}";
	  }
	else if (is_a<print_tree>(c)) 
	  {
	    inherited::print(c, level);
	  }
	else
	  {
	    if (precedence <= level) c.s << "(";
	    c.s << "Z(" ;
	    n.print(c);
	    c.s << ")";
	    if (precedence <= level) c.s << ")";
	  }
	return;
      }

    int i;

    if (is_a<print_latex>(c)) 
      {
	if (precedence <= level) c.s << "{(";
	c.s << "\\mbox{Li}_{" ;
	ex_to<letter>(letter_list.op(0)).get_degree().print(c,mul_precedence); 
	c.s << "}(" ;
	ex_to<letter>(letter_list.op(0)).get_scale().print(c); 
	c.s << ")";
	if (precedence <= level) c.s << ")}";
      }
    else if (is_a<print_tree>(c)) 
      {
	inherited::print(c, level);
      }
    else
      {
	if (precedence <= level) c.s << "(";
	c.s << "Li(" ;
	ex_to<letter>(letter_list.op(0)).get_degree().print(c,mul_precedence); 
	c.s << ",";
	ex_to<letter>(letter_list.op(0)).get_scale().print(c); 
	c.s << ")";
	if (precedence <= level) c.s << ")";
      }
  }

  /**
   *
   * No simplifications implemented for classical polylogs.
   *
   */
  ex classical_polylog::eval(int level) const
  {

    return this->hold();

  }

  // static member variables

  // same value as class power
  unsigned classical_polylog::precedence = 60;

} // namespace GiNaC

