
/** @file list_polygamma.h
 *
 * Interface to list_of_tgamma
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_LIST_POLYGAMMA_H__
#define __NESTEDSUMS_LIST_POLYGAMMA_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// The class list_of_tgamma is a container for the class ratio_of_tgamma.
  /**
   *
   *  The definition is 
   *  \f[
   *    \frac{\Gamma(1+d_1\varepsilon)}{\Gamma(1+b_1\varepsilon)}
   *    \frac{\Gamma(1+d_2\varepsilon)}{\Gamma(1+b_2\varepsilon)} ...
   *    \frac{\Gamma(1+d_k\varepsilon)}{\Gamma(1+b_k\varepsilon)}
   *  \f]
   *  \f[
   *     \frac{\Gamma(i+a_1+b_1\varepsilon)}{\Gamma(i+c_1+d_1\varepsilon)}
   *     \frac{\Gamma(i+a_2+b_2\varepsilon)}{\Gamma(i+c_2+d_2\varepsilon)} ...
   *     \frac{\Gamma(i+a_k+b_k\varepsilon)}{\Gamma(i+c_k+d_k\varepsilon)}
   *  \f]
   *
   */
  class list_of_tgamma : public basic
    {
      friend class transcendental_sum_type_A;
      friend class transcendental_sum_type_B;
      friend class transcendental_sum_type_C;
      friend class transcendental_sum_type_D;

      GINAC_DECLARE_REGISTERED_CLASS(list_of_tgamma, basic)

	// ctors
    public:
      explicit list_of_tgamma(unsigned ti);
      explicit list_of_tgamma(const ex & l);
      list_of_tgamma(const ex & l, const ex & i, const ex & eps, int o);

	// functions overriding virtual functions from base classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;
      ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;

    protected:
      ex simplify_ncmul(const exvector & v) const;
      ex derivative(const symbol & s) const;
      unsigned return_type(void) const { return return_types::noncommutative; }
      unsigned return_type_tinfo(void) const { return TINFO_list_of_tgamma; }
      unsigned calchash(void) const;

	// new virtual functions which can be overridden by derived classes
    public:
      virtual ex eval_explicit(int level = 0) const;
      virtual unsigned get_key(void) const;
      virtual ex hash_data(void) const;
      virtual ex subst_data(void) const;

	// non-virtual functions 
    public:
      ex set_expansion(void) const;
      ex set_index(const ex & new_index) const;
      ex set_expansion_parameter(const ex & new_eps) const;
      ex set_order(int new_order) const;

      ex get_index(void) const;
      unsigned get_depth(void) const;

      ex shift_plus_one(void) const;
      ex shift_minus_one(void) const;
      ex index_eq_one(void) const;
      ex shift_index(const ex & new_index) const;

      ex concat(const ex & l) const;

      int pole_alert(void) const;
      int expansion_alert(void) const;


      // friends :
      friend ex create_transcendental_sum_type_A_and_set_gammas(const ex & nn, const ex & i, 
				const ex & l, const ex & v, const ex & ss, 
				const ex & eps, int o, int f);
      friend ex create_transcendental_sum_type_B_and_set_gammas(const ex & nn, const ex & i, const ex & l, const ex & lr, 
				const ex & v, const ex & vr, const ex & ss, const ex & ssr, 
				const ex & eps, int o, int f);
      friend ex create_transcendental_sum_type_C_and_set_gammas(const ex & nn, const ex & i, const ex & l, 
				const ex & v, const ex & ss, 
				const ex & eps, int o, int f);
      friend ex create_transcendental_sum_type_D_and_set_gammas(const ex & nn, const ex & i, const ex & l, const ex & lr, 
				const ex & v, const ex & vr, const ex & ss, const ex & ssr, 
				const ex & eps, int o, int f);

	// member variables :

    protected:

      ex gamma_sequence;
      ex index;
      ex expansion_parameter;
      int order;

      static unsigned precedence;
    };

  // inline functions

  /**
   *
   * Returns the upper summation limit.
   *
   */
inline ex list_of_tgamma::get_index(void) const
  {
    return index;
  }

  /**
   *
   * Returns the depth.
   *
   */
inline unsigned list_of_tgamma::get_depth(void) const
  {
    return gamma_sequence.nops();
  }

  // utility functions
inline const list_of_tgamma &ex_to_list_of_tgamma(const ex &e)
  {
    return static_cast<const list_of_tgamma &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_list_of_tgamma(const ex & l)
  {
    return (new list_of_tgamma(l))->setflag(status_flags::dynallocated);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_list_of_tgamma(const ex & l, const ex & i, const ex & eps, int o)
  {
    return (new list_of_tgamma(l,i,eps,o))->setflag(status_flags::dynallocated);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_list_of_tgamma_and_set_gammas(const ex & l, const ex & i, const ex & eps, int o)
  {
    // set index, expansion_parameter and order in the Gamma functions
    lst* vl = new lst();
    vl->setflag(status_flags::dynallocated);
    int j;
    for (j=0;j<l.nops();j++)
      {
	vl->append(ex_to<ratio_of_tgamma>(l.op(j)).set_values(i,eps,o,expand_status::no_eval_to_scalar));
      }
    return (new list_of_tgamma(*vl,i,eps,o))->setflag(status_flags::dynallocated);
  }

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_LIST_POLYGAMMA_H__
