
/** @file multiple_zeta_value.h
 *
 * Interface to multiple zeta values
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_MULTIPLE_ZETA_VALUE_H__
#define __NESTEDSUMS_MULTIPLE_ZETA_VALUE_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// A multiple zeta value can be viewed as a special case of a multiple polylog 
  /// or as a special case of an Euler-Zagier sum.
  /**
   *
   *  Multiple zeta values are defined by
   *  \f[
   *     \zeta_{m_k,...,m_1} = Z(\infty;m_1,...,m_k;1,...,1) 
   *  \f]
   *
   */
  class multiple_zeta_value : public multiple_polylog, public Euler_Zagier_sum
    {
      GINAC_DECLARE_REGISTERED_CLASS(multiple_zeta_value, multiple_polylog)

	// ctors
    public:
      explicit multiple_zeta_value(unsigned ti);
      explicit multiple_zeta_value(const ex & llc);
      multiple_zeta_value(unsigned ti, const ex & llc);

	// functions overriding virtual functions from bases classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;

    protected:
      unsigned return_type(void) const { return return_types::noncommutative; }
      unsigned return_type_tinfo(void) const { return TINFO_multiple_polylog; }

	// member variables :

    protected:

      static unsigned precedence;
    };

  // utility functions
inline const multiple_zeta_value &ex_to_multiple_zeta_value(const ex &e)
  {
    return dynamic_cast<const multiple_zeta_value &>(*e.bp);
  }

  // template specialization
template<> inline const multiple_zeta_value &ex_to<multiple_zeta_value>(const ex &e)
  {
    return dynamic_cast<const multiple_zeta_value &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_multiple_zeta_value(const ex & llc)
  {
    return (new multiple_zeta_value(llc))->setflag(status_flags::dynallocated);
  }

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_MULTIPLE_ZETA_VALUE_H__
