
/** @file nielsen_polylog.h
 *
 * Interface to Nielsen's generalized polylogarithms 
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_NIELSEN_POLYLOG_H__
#define __NESTEDSUMS_NIELSEN_POLYLOG_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// A Nielsen polylog is a special case of a harmonic polylog.
  /**
   *
   *  Nielsen polylogs are defined by
   *  \f[
   *     \mbox{S}_{n,p}(x) = \mbox{H}_{n+1,1,...,1}(x) 
   *  \f]
   *  where (p-1) one's occur in the index field.
   *
   */
  class nielsen_polylog : public harmonic_polylog
    {
      GINAC_DECLARE_REGISTERED_CLASS(nielsen_polylog, harmonic_polylog)

	// ctors
    public:
      explicit nielsen_polylog(unsigned ti);
      explicit nielsen_polylog(const ex & llc);
      nielsen_polylog(unsigned ti, const ex & llc);

	// functions overriding virtual functions from bases classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;

	// member variables :

    protected:

      static unsigned precedence;
    };

  // utility functions
inline const nielsen_polylog &ex_to_nielsen_polylog(const ex &e)
  {
    return dynamic_cast<const nielsen_polylog &>(*e.bp);
  }

  // template specialization
template<> inline const nielsen_polylog &ex_to<nielsen_polylog>(const ex &e)
  {
    return dynamic_cast<const nielsen_polylog &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_nielsen_polylog(const ex & llc)
  {
    return (new nielsen_polylog(llc))->setflag(status_flags::dynallocated);
  }

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_NIELSEN_POLYLOG_H__
