
/** @file polygamma.h
 *
 * Interface to ratio_of_tgamma
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_POLYGAMMA_H__
#define __NESTEDSUMS_POLYGAMMA_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// The class ratio_of_tgamma contains the ratio of two Gamma functions
  /**
   *
   *  The definition is 
   *  \f[
   *     \frac{\Gamma(1+b_2\varepsilon)}{\Gamma(1+b_1\varepsilon)}
   *     \frac{\Gamma(i+a_1+b_1\varepsilon)}{\Gamma(i+a_2+b_2\varepsilon)}
   *  \f]
   * Here \f$ a_1 \f$, \f$ b_1 \f$, \f$ a_2 \f$ and \f$ b_2 \f$ are passed to the constructor.
   * The index \f$i\f$ and the expansion parameter \f$\varepsilon\f$ can be set via the
   * set_index(const ex &)  and set_expansion_parameter(const ex &) methods, respectively.
   *
   * Additionally, this class contains an internal variable "order", which indicates
   * to which order this object should be expanded. The "order" variable can be set via
   * the method set_order(int). The expansion is performed by calling the method
   * set_expansion(void).
   *
   * This class assumes that \f$i+a_1 > 0\f$ for all \f$i\f$, e.g. it does not handle
   * possible poles in the Gamma function.
   *
   */
  class ratio_of_tgamma : public basic
    {
      friend class list_of_tgamma;

      GINAC_DECLARE_REGISTERED_CLASS(ratio_of_tgamma, basic)

	// ctors
    public:
      explicit ratio_of_tgamma(unsigned ti);
      ratio_of_tgamma(const ex & a1, const ex & b1, const ex & a2, const ex & b2);
      ratio_of_tgamma(const ex & a1, const ex & b1, const ex & a2, const ex & b2, const ex & i, const ex & eps, 
        int o, int f);

	// functions overriding virtual functions from base classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;
      ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;

    protected:
      ex simplify_ncmul(const exvector & v) const;
      ex derivative(const symbol & s) const;
      unsigned return_type(void) const { return return_types::noncommutative; }
      unsigned return_type_tinfo(void) const { return TINFO_ratio_of_tgamma; }
      unsigned calchash(void) const;

	// new virtual functions which can be overridden by derived classes
    public:
      virtual ex eval_explicit(int level = 0) const;

	// non-virtual functions 
    public:
      ex set_expansion(void) const;
      ex set_index(const ex & new_index) const;
      ex set_expansion_parameter(const ex & new_eps) const;
      ex set_order(int new_order) const;
      ex set_flag(int new_flag) const;
      ex set_values(const ex & new_index, const ex & new_eps, int new_order, int new_flag) const;

      ex get_index(void) const;

      ex shift_plus_one(void) const;
      ex shift_minus_one(void) const;
      ex index_eq_one(void) const;
      ex shift_index(const ex & new_index) const;

      int pole_alert(void) const;
      int expansion_alert(void) const;

    protected:
      ex increase_numerator(int shift_order = 0) const;
      ex decrease_numerator(int shift_order = 0) const;
      ex increase_denominator(int shift_order = 0) const;
      ex decrease_denominator(int shift_order = 0) const;

      ex prefactor_increase_numerator(void) const;
      ex prefactor_decrease_numerator(void) const;
      ex prefactor_increase_denominator(void) const;
      ex prefactor_decrease_denominator(void) const;

	// member variables :

    protected:

      ex a1;
      ex b1;
      ex a2;
      ex b2;

      ex index;
      ex expansion_parameter;

      int order;
      int flag_expand_status;

      static unsigned precedence;
    };

  // utility functions
inline const ratio_of_tgamma &ex_to_ratio_of_tgamma(const ex &e)
  {
    return static_cast<const ratio_of_tgamma &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_ratio_of_tgamma(const ex & a1, const ex & b1, const ex & a2, const ex & b2)
  {
      return (new ratio_of_tgamma(a1,b1,a2,b2))->setflag(status_flags::dynallocated);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_ratio_of_tgamma(const ex & a1, const ex & b1, const ex & a2, const ex & b2, const ex & i, const ex & eps, 
        int o, int f)
  {
    return (new ratio_of_tgamma(a1,b1,a2,b2,i,eps,o,f))->setflag(status_flags::dynallocated);
  }

 ex eval_ratio_of_tgamma_to_scalar(const ex & expr);

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_POLYGAMMA_H__
