
/** @file transc_fct_C.h
 *
 * Interface to transcendental_fct_type_C
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_TRANSC_FCT_C_H__
#define __NESTEDSUMS_TRANSC_FCT_C_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// The class transcendental_fct_type_C provides an interface
  /// to the class transcendental_sum_type_C.
  /**
   *
   *  The definition is 
   *  \f[
   *     \frac{\Gamma(d_1) ... \Gamma(d_n)}{\Gamma(d_1') ... \Gamma(d_{n'}')} 
   *      \sum\limits_{i=0}^\infty \sum\limits_{j=0}^\infty 
   *       \frac{\Gamma(i+a_1) ... \Gamma(i+a_k)}{\Gamma(i+a_1') ... \Gamma(i+a_{k}')} 
   *       \frac{\Gamma(i+j+c_1) ... \Gamma(i+j+c_m)}{\Gamma(i+j+c_1') ... \Gamma(i+j+c_{m-1}')} 
   *      \frac{x_1^i}{i!} \frac{x_2^j}{j!}
   *  \f]
   *
   */
  class transcendental_fct_type_C : public basic
    {
      GINAC_DECLARE_REGISTERED_CLASS(transcendental_fct_type_C, basic)

	// ctors
    public:
      explicit transcendental_fct_type_C(unsigned ti);
      transcendental_fct_type_C(const ex & xx1, const ex & xx2, const ex & ii_num, const ex & ii_denom, 
				const ex & iijj_num, const ex & iijj_denom, 
				const ex & pp_num, const ex & pp_denom);
      transcendental_fct_type_C(const ex & xx1, const ex & xx2, const ex & ii_num, const ex & ii_denom, 
				const ex & iijj_num, const ex & iijj_denom, 
				const ex & pp_num, const ex & pp_denom, const ex & eps, int o, int f);

	// functions overriding virtual functions from base classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;
      ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;

    protected:
      ex simplify_ncmul(const exvector & v) const;
      ex derivative(const symbol & s) const;
      unsigned return_type(void) const { return return_types::noncommutative; }
      unsigned return_type_tinfo(void) const { return TINFO_transcendental_fct_type_C; }
      unsigned calchash(void) const;

	// non-virtual functions 
    public:
      ex set_expansion(const ex & eps, int o) const;
      ex shift_plus_one(void) const;

	// member variables :

    protected:

      ex x1;
      ex x2;
      ex i_num;
      ex i_denom;
      ex ij_num;
      ex ij_denom;
      ex pre_num;
      ex pre_denom;

      ex expansion_parameter;

      int order;

      mutable int flag_expand_status;

      static unsigned precedence;
    };

  // utility functions
inline const transcendental_fct_type_C &ex_to_transcendental_fct_type_C(const ex &e)
  {
    return static_cast<const transcendental_fct_type_C &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_transcendental_fct_type_C(const ex & xx1, const ex & xx2, const ex & ii_num, const ex & ii_denom,
					   const ex & iijj_num, const ex & iijj_denom, 
					   const ex & pp_num, const ex & pp_denom)
  {
    return (new transcendental_fct_type_C(xx1,xx2,ii_num,ii_denom,iijj_num,iijj_denom,pp_num,pp_denom)
	    )->setflag(status_flags::dynallocated);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_transcendental_fct_type_C(const ex & xx1, const ex & xx2, const ex & ii_num, const ex & ii_denom, 
					   const ex & iijj_num, const ex & iijj_denom, 
					   const ex & pp_num, const ex & pp_denom, const ex & eps, int o, int f)
  {
    return (new transcendental_fct_type_C(xx1,xx2,ii_num,ii_denom,iijj_num,iijj_denom,pp_num,pp_denom,eps,o,f)
	    )->setflag(status_flags::dynallocated);
  }

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_TRANSC_FCT_C_H__
