
/** @file transcendental_B.h
 *
 * Interface to transcendental_sum_type_B
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_TRANSCENDENTAL_B_H__
#define __NESTEDSUMS_TRANSCENDENTAL_B_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// The class transcendental_sum_type_B may contain \f$i\f$ and \f$(n-i)\f$.
  /**
   *
   *  The definition is 
   *  \f[
   *    \frac{\Gamma(1+d_1\varepsilon)}{\Gamma(1+b_1\varepsilon)} ...
   *    \frac{\Gamma(1+d_k\varepsilon)}{\Gamma(1+b_k\varepsilon)}
   *    \frac{\Gamma(1+d_1'\varepsilon)}{\Gamma(1+b_1'\varepsilon)} ...
   *    \frac{\Gamma(1+d_{k'}'\varepsilon)}{\Gamma(1+b_{k'}'\varepsilon)}
   *  \f]
   *  \f[
   *     \sum\limits_{i=1}^{n-1} 
   *       \frac{x^i}{(i+c)^m} 
   *       \frac{\Gamma(i+a_1+b_1\varepsilon)}{\Gamma(i+c_1+d_1\varepsilon)}
   *       \frac{\Gamma(i+a_2+b_2\varepsilon)}{\Gamma(i+c_2+d_2\varepsilon)} ...
   *       \frac{\Gamma(i+a_k+b_k\varepsilon)}{\Gamma(i+c_k+d_k\varepsilon)}
   *       Z(i+o-1,m_1,...,m_l,x_1,...,x_l)
   *  \f]
   *  \f[
   *       \cdot
   *       \frac{y^{n-i}}{(n-i+c')^{m'}} 
   *       \frac{\Gamma(n-i+a_1'+b_1'\varepsilon)}{\Gamma(n-i+c_1'+d_1'\varepsilon)}
   *       \frac{\Gamma(n-i+a_2'+b_2'\varepsilon)}{\Gamma(n-i+c_2'+d_2'\varepsilon)} ...
   *       \frac{\Gamma(n-i+a_{k'}'+b_{k'}'\varepsilon)}{\Gamma(n-i+c_{k'}'+d_{k'}'\varepsilon)}
   *       Z(n-i+o'-1,m_1',...,m_{l'}',x_1',...,x_{l'}')
   *  \f]
   *
   * Here, \f$c\f$, \f$c'\f$, \f$o\f$, \f$o'\f$ and all \f$a_j\f$, \f$a_j'\f$, \f$c_j\f$  and \f$c_j'\f$ are integers.
   * In addition \f$c\f$ is non-negative or \f$ c \ge -l+o \f$.
   * Simmilar, \f$c'\f$ is non-negative or \f$ c' \ge -l'+o' \f$.
   *
   * Note that the upper summation limit is \f$(n-1)\f$.
   * The upper summation limit should not be infinity.
   *
   */
  class transcendental_sum_type_B : public basic
    {
      GINAC_DECLARE_REGISTERED_CLASS(transcendental_sum_type_B, basic)

	// ctors
    public:
      explicit transcendental_sum_type_B(unsigned ti);
      transcendental_sum_type_B(const ex & nn, const ex & i, const ex & l, const ex & lr, 
			       const ex & v, const ex & vr, const ex & ss, const ex & ssr, 
			       const ex & eps, int o, int f);

	// functions overriding virtual functions from base classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;
      ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;

    protected:
      ex simplify_ncmul(const exvector & v) const;
      ex derivative(const symbol & s) const;
      unsigned return_type(void) const { return return_types::noncommutative; }
      unsigned return_type_tinfo(void) const { return TINFO_transcendental_sum_type_B; }
      unsigned calchash(void) const;

	// new virtual functions which can be overridden by derived classes
    public:
      virtual ex eval_explicit(int level = 0) const;

	// non-virtual functions 
    public:
      ex set_expansion(void) const;
      ex distribute_over_subsum(void) const;
      ex distribute_over_letter(void) const;
      ex distribute_over_subsum_rev(void) const;
      ex distribute_over_letter_rev(void) const;
      ex shift_plus_one(void) const;
      ex shift_plus_one_rev(void) const;

    protected:

	// member variables :

    protected:

      ex n;
      ex index;
      ex letter;
      ex letter_rev;
      ex lst_of_gammas;
      ex lst_of_gammas_rev;
      ex subsum;
      ex subsum_rev;

      ex expansion_parameter;

      int order;

      mutable int flag_expand_status;

      static unsigned precedence;
    };

  // utility functions
inline const transcendental_sum_type_B &ex_to_transcendental_sum_type_B(const ex &e)
  {
    return static_cast<const transcendental_sum_type_B &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_transcendental_sum_type_B(const ex & nn, const ex & i, const ex & l, const ex & lr, 
				    const ex & v, const ex & vr, const ex & ss, const ex & ssr, 
				    const ex & eps, int o, int f)
  {
    return (new transcendental_sum_type_B(nn,i,l,lr,v,vr,ss,ssr,eps,o,f))->setflag(status_flags::dynallocated);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_transcendental_sum_type_B_and_set_gammas(const ex & nn, const ex & i, const ex & l, const ex & lr, 
				    const ex & v, const ex & vr, const ex & ss, const ex & ssr, 
				    const ex & eps, int o, int f)
  {
    // set index, expansion_parameter and order in the Gamma functions
    ex new_v = create_list_of_tgamma_and_set_gammas(ex_to<list_of_tgamma>(v).gamma_sequence,i,eps,o);

    // set index, expansion_parameter and order in the Gamma functions
    ex new_vr = create_list_of_tgamma_and_set_gammas(ex_to<list_of_tgamma>(vr).gamma_sequence,nn-i,eps,o);

    return (new transcendental_sum_type_B(nn,i,l,lr,new_v,new_vr,ss,ssr,eps,o,f))->setflag(status_flags::dynallocated);
  }

ex create_transcendental_sum_type_B_and_expand(const ex & nn, const ex & i, const ex & l, const ex & lr, 
					       const ex & v, const ex & vr, const ex & ss, const ex & ssr, 
					       const ex & eps, int o);

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_TRANSCENDENTAL_B_H__
